package inettools;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import javax.microedition.lcdui.Font;
import javax.microedition.rms.RecordStore;

import lib.Utils.Utils;

public class Database {
	// these are set in setprofile that is always called
	String profilename;
	String nick;
	String altnick;
	String host;
	int port;
	String channels;
	String username;
	String realname;
	String passwd;
	String start_commands;
	String quit_message;


	int[] idxarray;

	// other options
	int profileidx = -1;
	boolean smiles=true;
	boolean timestamp=false;
	boolean showtime=true;
	boolean usecolor=true;
	boolean showinput=false;
	boolean showhosts=false;
	boolean ajoinkick=true;
	boolean poolmode=true;
	int pooltime = 20;
	int clockpos = 0;

	boolean vibra=false;
	boolean sound=false;
	boolean light=false;
	
	boolean onquery=false;
	boolean onaddress=false;
	boolean onnotice=false;
	boolean onaction=false;

	int userId = 0;
	int fontsize=Font.SIZE_SMALL;
	
	String encoding = "Windows-1251";
	String ctcp_info = Irc.language.get("userinfovalue");
	String template_query = "%bold%%color%8,2%nick%%color%%bold%, ";
	int buflines = 15;
	int height_diff = 0;


	private final static String STORE_CONFIG = "irccfg";
	private final static String STORE_PROFILE = "ircprof";
	
	public Database() {
	}

	/**
	 * returns 
	 */
	public void load() {
		DataInputStream din;

		try {
			RecordStore rs = RecordStore.openRecordStore(STORE_CONFIG, true);

			int size = rs.getNumRecords();
			String version;

			try {
				byte[] temp = rs.getRecord(1);
				version = (new DataInputStream(new ByteArrayInputStream(temp))).readUTF();
			} catch (Exception e) {
				version = "";
			}

			if (!version.equals(Irc.VERSION)) {
				rs.closeRecordStore();
				try {
					RecordStore.deleteRecordStore(STORE_CONFIG);
					RecordStore.deleteRecordStore(STORE_PROFILE);
				} catch (Exception e) {}

				rs = RecordStore.openRecordStore(STORE_CONFIG, true);

				byte[] temp;
				ByteArrayOutputStream baos = new ByteArrayOutputStream();
				DataOutputStream dos = new DataOutputStream(baos);
				dos.writeUTF(Irc.VERSION);
				temp = baos.toByteArray();
				dos.close();

				rs.addRecord(temp, 0, temp.length);
				temp = new byte[0];
				rs.addRecord(temp, 0, temp.length);
				rs.addRecord(temp, 0, temp.length);
				rs.addRecord(temp, 0, temp.length);

				save_profile();
				save_advanced();
				profileidx = 0;
			}
			else {
				din = new DataInputStream(new ByteArrayInputStream(rs.getRecord(2)));
				profileidx = din.readInt();
				
				din = new DataInputStream(new ByteArrayInputStream(rs.getRecord(3)));
				timestamp = din.readBoolean();
				usecolor = din.readBoolean();
				smiles = din.readBoolean();
				showinput = din.readBoolean();
				showhosts = din.readBoolean();
				ajoinkick = din.readBoolean();
				userId = din.readInt();

				vibra = din.readBoolean();
				sound = din.readBoolean();
				light = din.readBoolean();

				onquery = din.readBoolean();
				onaddress = din.readBoolean();
				onnotice = din.readBoolean();
				onaction = din.readBoolean();

				fontsize = din.readInt();
				encoding = din.readUTF();
				height_diff = din.readInt();
				buflines = din.readInt();
				ctcp_info = din.readUTF();
				template_query = din.readUTF();

				showtime = din.readBoolean();
				poolmode = din.readBoolean();
				pooltime = din.readInt();
				clockpos = din.readInt();

				din.close();
			}
			rs.closeRecordStore();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
		getProfiles();
		setProfile(profileidx);
	}
	
	public String[] getProfiles() {
		String[] ret = null;

		try {
			byte[] temp;
			DataInputStream din;
			RecordStore rs = RecordStore.openRecordStore(STORE_PROFILE, true);

			if (rs.getNumRecords() == 0) {
				rs.addRecord(new byte[4], 0, 4);
				idxarray = new int[0];
				ret = new String[0];
			}
			else {
				din = new DataInputStream(new ByteArrayInputStream(rs.getRecord(1)));
				rs.closeRecordStore();

				int profiles = din.readInt();
				ret = new String[profiles];
				idxarray = new int[profiles];

				for (int i=0; i<ret.length; i++) {
					ret[i] = din.readUTF();
					idxarray[i] = din.readInt();
				}
				din.close();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return ret;
	}

	public void setProfile(int index) {
		if (index < 0) {
			profilename = "Default";
			nick = "";
			altnick = "";
			host =  "irc.tic-tac.ru";
			port = 6667;
			channels = "#inettools;#j2me";
			username = "";
			realname = "";
			passwd = "";
			start_commands = "";
			quit_message="InetTools IRC "+Irc.VERSION+"   .";
		}
		else if (index < idxarray.length) {
			try {
				int rsidx = idxarray[index];
				RecordStore rs = RecordStore.openRecordStore(STORE_PROFILE, false);
				if (rsidx > 0) {
					DataInputStream dis = new DataInputStream(new ByteArrayInputStream(rs.getRecord(rsidx)));

					profilename = dis.readUTF();
					nick = dis.readUTF();
					altnick = dis.readUTF();
					host = dis.readUTF();
					port = dis.readInt();
					channels = dis.readUTF();
					username = dis.readUTF();
					realname = dis.readUTF();
					passwd = dis.readUTF();
					start_commands = dis.readUTF();
					quit_message = dis.readUTF();
				}
				rs.closeRecordStore();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}


	public void addProfile() {
		// this should handle the adding also just fine
		editProfile(idxarray.length);
	}

	public void editProfile(int index) {
		try {
			ByteArrayOutputStream baos;
			DataOutputStream dos;

			RecordStore rs = RecordStore.openRecordStore(STORE_PROFILE, false);
			editProfileName(rs, index, profilename);

			baos = new ByteArrayOutputStream();
			dos = new DataOutputStream(baos);
			dos.writeUTF(profilename);
			dos.writeUTF(nick);
			dos.writeUTF(altnick);
			dos.writeUTF(host);
			dos.writeInt(port);
			dos.writeUTF(channels);
			dos.writeUTF(username);
			dos.writeUTF(realname);
			dos.writeUTF(passwd);
			dos.writeUTF(start_commands);
			dos.writeUTF(quit_message);
			byte[] temp = baos.toByteArray();
			dos.close();

			rs.setRecord(idxarray[index], temp, 0, temp.length);
			rs.closeRecordStore();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public void deleteProfile(int index) {
		try {
			RecordStore rs = RecordStore.openRecordStore(STORE_PROFILE, false);

			rs.deleteRecord(idxarray[index]);
			editProfileName(rs, index, null); // delete profile name

			rs.closeRecordStore();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void editProfileName(RecordStore rs, int idx, String newname) throws Exception {
		int i, profiles;
		DataInputStream dis;
		DataOutputStream dos;
		ByteArrayOutputStream baos;
		boolean createnew = false;

		byte[] temp = rs.getRecord(1);
		dis = new DataInputStream(new ByteArrayInputStream(temp));
		baos = new ByteArrayOutputStream();
		dos = new DataOutputStream(baos);

		profiles = dis.readInt();
		if (newname == null && idx >= 0 && idx < profiles) {
			// if we delete a profile then decrease the idxarray size
			profiles--;;
			idxarray = new int[profiles];
		}
		else if (idx < 0 || idx >= profiles) {
			// create a new profile
			profiles++;
			idxarray = new int[profiles];
			createnew = true;
		}
		else if (newname == null) {
			// can't delete a profile that doesn't exist
			return;
		}
		dos.writeInt(profiles);

		for (i=0; i<profiles; i++) {
			if (i == profiles-1 && createnew) break;

			if (i == idx) {
				if (newname != null) {
					dis.readUTF();
					dos.writeUTF(newname);
					idxarray[i] = dis.readInt();
					dos.writeInt(idxarray[i]);
					continue;
				}
				else {
					dis.readUTF();
					dis.readInt();
				}
			}
			dos.writeUTF(dis.readUTF());
			idxarray[i] = dis.readInt();
			dos.writeInt(idxarray[i]);
		}
		if (createnew) {
			// add a new profile to the end
			dos.writeUTF(newname);
			idxarray[i] = rs.getNextRecordID();
			dos.writeInt(idxarray[i]);

			// also add the empty record for it
			rs.addRecord(new byte[0], 0, 0);
		}

		temp = baos.toByteArray();
		dis.close();
		dos.close();
		rs.setRecord(1, temp, 0, temp.length);

		if (profileidx >= idxarray.length)
			profileidx = idxarray.length-1;
	}

	public String[] getChannels() {
		return Utils.hasNoValue(channels) ? null : Utils.splitString(channels, ";");
	}
	
	public String[] getStartCommands() {
		return Utils.hasNoValue(start_commands) ? null : Utils.splitString(start_commands, ";");
	}
	
	public void save_profile() {
		try{
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			DataOutputStream dos = new DataOutputStream(baos);
			byte[] byteout;

			dos.writeInt(profileidx);

			byteout = baos.toByteArray();
			dos.close();
			baos.close();

			RecordStore rs = RecordStore.openRecordStore(STORE_CONFIG, true);
			rs.setRecord(2, byteout, 0, byteout.length);
			rs.closeRecordStore();
		}
		catch(Exception e) {
			e.printStackTrace();
			// error in saving settings, should handle this
		}
	}

	public void save_advanced() {
		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			DataOutputStream dos = new DataOutputStream(baos);
			byte[] byteout;

			dos.writeBoolean(timestamp);
			dos.writeBoolean(usecolor);
			dos.writeBoolean(smiles);
			dos.writeBoolean(showinput);
			dos.writeBoolean(showhosts);
			dos.writeBoolean(ajoinkick);
			dos.writeInt(userId);

			dos.writeBoolean(vibra);
			dos.writeBoolean(sound);
			dos.writeBoolean(light);
			
			dos.writeBoolean(onquery);
			dos.writeBoolean(onaddress);
			dos.writeBoolean(onnotice);
			dos.writeBoolean(onaction);
			
			dos.writeInt(fontsize);
			dos.writeUTF(encoding);
			dos.writeInt(height_diff);
			dos.writeInt(buflines);
			dos.writeUTF(ctcp_info);
			dos.writeUTF(template_query);

			dos.writeBoolean(showtime);
			dos.writeBoolean(poolmode);
			dos.writeInt(pooltime);
			dos.writeInt(clockpos);

			byteout = baos.toByteArray();
			dos.close();
			baos.close();

			RecordStore rs = RecordStore.openRecordStore(STORE_CONFIG, true);
			rs.setRecord(3, byteout, 0, byteout.length);
			rs.closeRecordStore();
		}
		catch(Exception e) {
			e.printStackTrace();
			// error in saving settings, should handle this
		}
	}


}
