package inettools;

import java.util.Calendar;
import java.util.Enumeration;
import java.util.Hashtable;

import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Font;

import lib.GraphicControls.MessageBox;
import lib.Utils.Utils;

public class Listener extends Thread {
	private IrcConnection irc;
	private UIHandler uihandler;
	private String whois = "";
	private Display display;
	private MessageBox mb;
	private boolean needupdate, nicktried, showinput, showhosts, ajoinkick, poolmode;
	
	private boolean vibra, sound, light, onnotice;

	private String altnick, host, idpass, password, username, realname, info;
	private int port, pooltime;
	private String[] channels;
	private String[] start_commands;
	
	public Listener(Database db, IrcConnection irc, UIHandler uihandler) {
		this.irc = irc;
		this.uihandler = uihandler;

		this.altnick = db.altnick;
		this.host = db.host;
		this.password = db.passwd;
		this.username = db.username;
		this.realname = db.realname;
		this.port = db.port;
		this.showinput = db.showinput;
		this.showhosts = db.showhosts;
		this.ajoinkick = db.ajoinkick;
		channels = db.getChannels();
		start_commands = db.getStartCommands();
		info = db.ctcp_info;
		
		this.vibra = db.vibra;
		this.sound = db.sound;
		this.light = db.light;
		this.onnotice = db.onnotice;
		
		this.poolmode = db.poolmode;
		this.pooltime = db.pooltime;

		//whois = new Hashtable();
		needupdate = false;
		nicktried = false;

		
}
	
	public void run() {
		String initstr = "", ret;

		String host = this.host;
		int port = this.port;
		uihandler.getConsole().writeInfo(Irc.language.get("connectto")+" " + host + ":" + port);

		String tmpnick = UIHandler.nick.equals("") ? "Irc_usr" : UIHandler.nick;
		String tmpuser = username.equals("") ? tmpnick : username;
		String tmpreal = realname.equals("") ? "Irc user" : realname;

		if (!Utils.hasNoValue(this.password))
			initstr += "PASS " + this.password + "\r\n";
			initstr += "NICK " + tmpnick + "\r\n";
			initstr += "USER " + tmpuser + " 8 * :" + tmpreal + "\r\n";

		ret = irc.connect(host, port, initstr);

		if (ret == null)
			listen();
		else
			uihandler.getConsole().writeInfo(ret);

		uihandler.clearChanPriv();
	}

	private void listen() {
		String data, ret = null;
		int counter;

		counter = 0;
		while(irc.isConnected()) {
			try {Thread.sleep(500);} catch(InterruptedException ie){}
			counter++;

			// check poll time here
			if (poolmode && counter >= pooltime*2) {
				needupdate = true;
			}

			if (needupdate) {
				ret = irc.updateConnection();
				needupdate = false;
				counter = 0;
			}

			if (ret == null) {
				while (irc.hasDataInBuffer()) {
					data = irc.readLine();
					if (data != null && !data.trim().equals("")) checkMessage(data);
				}
			}
			if (ret != null) {
				uihandler.getConsole().writeInfo(ret);
				ret = null;
			}
		}
		uihandler.getConsole().writeInfo(Irc.language.get("disconnecting"));
	}
	
	private void checkMessage(String response) {
		String[] cmdline, command;
		String fromnick, firstparam;
		Window ch;
		char temp;
		int cmdnum;


		try{
			cmdline = parseLine(response);

			if (cmdline[1] == null) return;

			command = Utils.splitString(cmdline[1], " ");
			if (command[0].equals("PING")) {
				Irc.writeLine("PONG :" + cmdline[2]);
				return;
			}

			if (cmdline[0] == null) return;
			if (cmdline[0].indexOf('!') >= 0)
				fromnick = cmdline[0].substring(0, cmdline[0].indexOf('!'));
			else fromnick = cmdline[0];

			if (cmdline[2] == null) firstparam = "";
			else if (cmdline[2].indexOf(" ") != -1)
				firstparam = cmdline[2].substring(0, cmdline[2].indexOf(" "));
			else
				firstparam = cmdline[2];

			try { cmdnum = Integer.parseInt(command[0]); }
			catch (NumberFormatException nfe) { cmdnum = 0; }

			
			if (cmdnum == 0) {
				if (command[0].equals("MODE")) {
					if (Utils.isChannel(command[1]) && command.length >= 3) {
						String[] nicks = new String[3];
						String chaninfo;
						Window win;
						int counter = 0;
						boolean adding = true;

						win = uihandler.getChannel(command[1]);
						nicks[2] = (command.length==5)?cmdline[2]:null;
						nicks[1] = (command.length==5)?command[4]:null;
						nicks[1] = (command.length==4)?cmdline[2]:nicks[1];
						nicks[0] = (command.length>=4)?command[3]:cmdline[2];

						for (int i=0; i<command[2].length(); i++) {
							switch(command[2].charAt(i)) {
								case '+':
									adding = true;
									break;
								case '-':
									adding = false;
									break;
								case 'o':
									win.changeMode(Window.MODE_OP, nicks[counter], adding);
									counter++;
									break;
								case 'h':
									win.changeMode(Window.MODE_HALFOP, nicks[counter], adding);
									counter++;
									break;
								case 'v':
									win.changeMode(Window.MODE_VOICE, nicks[counter], adding);
									counter++;
									break;
							}
							if (counter > 2 || nicks[counter] == null) break;
						}
						chaninfo = "* " + fromnick +" "+ Irc.language.get("changemode")+"'" + command[2] + "' ";
						for (int i=0; i < 3 && nicks[i] != null; i++)
							chaninfo += " " + nicks[i];

						win.writeAction(chaninfo);
					}
				}
				else if (command[0].equals("PRIVMSG")) {
					temp = command[1].charAt(0);
					if (cmdline[2].indexOf('\001') != -1) {
						// found a ctcp command in line, parse all tagged data from string
						int ctcpidx1, ctcpidx2 = -1;
						while((ctcpidx1 = cmdline[2].indexOf('\001', ctcpidx2+1)) != -1) {
							if ((ctcpidx2 = cmdline[2].indexOf('\001', ctcpidx1+1)) == -1)
								break;
							String ctcpcmd;
							String ctcpbody = cmdline[2].substring(ctcpidx1+1, ctcpidx2);
							if (ctcpbody.indexOf(' ') != -1) {
								ctcpcmd = ctcpbody.substring(0, ctcpbody.indexOf(' ')).toUpperCase();
								ctcpbody = ctcpbody.substring(ctcpbody.indexOf(' '));
							}
							else {
								ctcpcmd = ctcpbody.toUpperCase();
								ctcpbody = "";
							}
							if (ctcpcmd.equals("ACTION")) {
								if (temp == '#' || temp == '&' || temp == '!') {
									uihandler.getChannel(command[1]).writeAction("* " + fromnick + " " + ctcpbody);
								}
								else {
									uihandler.getPrivate(fromnick).writeAction("* " + fromnick + " " + ctcpbody);
								}
							}
							else if (ctcpcmd.equals("PING")) {
								Irc.writeLine("NOTICE " + fromnick + " :\001PING" + ctcpbody + "\001");
								uihandler.getConsole().writeAction("-" + fromnick + "- "+Irc.language.get("pingingyou"));
							}
							else if (ctcpcmd.equals("TIME")) {
								Calendar cal = Calendar.getInstance();
								Irc.writeLine("NOTICE " + fromnick + " :\001TIME " + "[" + cal.get(Calendar.HOUR_OF_DAY) + ":" +(cal.get(Calendar.MINUTE)<10?"0":"") + cal.get(Calendar.MINUTE) + "]" + "\001");
								uihandler.getConsole().writeAction("-" + fromnick + "- "+Irc.language.get("checkyoutime"));
							}
							else if (ctcpcmd.equals("USERINFO")) {
								Irc.writeLine("NOTICE " + fromnick + " :\001USERINFO " + info + "\001");
								uihandler.getConsole().writeAction("-" + fromnick + "- "+Irc.language.get("checkyouinfo"));
							}
							else if (ctcpcmd.equals("VERSION")) {
								String platf = System.getProperty("microedition.platform");
								if (platf == null) platf = "J2ME device";

								Irc.writeLine("NOTICE " + fromnick + " :\001VERSION InetTools v" + Irc.VERSION + " on " + platf + "\001");
								uihandler.getConsole().writeAction("-" + fromnick + "- "+Irc.language.get("checkyouversion"));
							}
							else uihandler.getConsole().writeInfo("* "+Irc.language.get("unknownctcp")+" \'" + ctcpcmd + "\' "+Irc.language.get("from")+ " " + fromnick + ":" + ctcpbody);
						}
					}
					else if (temp == '#' || temp == '&' || temp == '!') {
						uihandler.getChannel(command[1]).write(fromnick, cmdline[2]);
					}
					else {
						uihandler.getPrivate(fromnick).write(fromnick, cmdline[2]);
					}
				}
				else if (command[0].equals("NOTICE")) {
					//FIXME
					if(cmdline[2].indexOf("\001PING") > -1){
						uihandler.getConsole().writeAction("-" + fromnick + "- PING "+((System.currentTimeMillis()-Long.parseLong(cmdline[2].substring(cmdline[2].indexOf("PING")+5,cmdline[2].length()-1)))/1000)+" "+Irc.language.get("seconds"));
					}else if(cmdline[2].indexOf("\001")>-1){
						uihandler.getConsole().writeAction("-" + fromnick  + "- " + cmdline[2].substring(cmdline[2].indexOf("\001")+1, cmdline[2].substring(cmdline[2].indexOf("\001")+1).indexOf("\001")+1));
					}
					else uihandler.getConsole().writeAction("-" + fromnick + "- " + cmdline[2]);
					if(onnotice){
						if(sound) Resources.playAction();
						if(light) Resources.LCDFlash();
						if(vibra) Resources.vibrate();
					}

				}
				else if (command[0].equals("NICK")) {
					String str = fromnick + " " +Irc.language.get("changenick")+" " + firstparam;
					Hashtable ht = uihandler.getChannels();
					Enumeration en = ht.elements();
					while (en.hasMoreElements()) {
						ch = (Window) en.nextElement();
						if (ch.hasNick(fromnick)) {
							ch.writeInfo(str);
							ch.changeNick(fromnick, firstparam);
						}
					}
					if(uihandler.privates.get(fromnick) != null){
						Window p = (Window) uihandler.privates.get(fromnick);
						uihandler.privates.remove(fromnick);
						p.header = firstparam;
						uihandler.privates.put(p.header, p);
						uihandler.setDisplay(p);
						p.repaint();
					}
				}
				else if (command[0].equals("QUIT")) {
					Hashtable ht = uihandler.getChannels();
					Enumeration en = ht.elements();
					while(en.hasMoreElements()) {
						ch = (Window) en.nextElement();
						if (ch.hasNick(fromnick)) {
							if(showhosts)ch.writeInfo(fromnick +" ("+cmdline[0].substring(cmdline[0].indexOf('!')+1)+") " +Irc.language.get("quit")+ " (" + cmdline[2] + ")");
							else ch.writeInfo(fromnick +" " +Irc.language.get("quit")+ " (" + cmdline[2] + ")");
							ch.deleteNick(fromnick);
						}
					}
				}
				else if (command[0].equals("JOIN")) {
					ch = uihandler.getChannel(firstparam);
					if(showhosts)ch.writeInfo(fromnick+" ("+cmdline[0].substring(cmdline[0].indexOf('!')+1)+") " +Irc.language.get("join")+ " " + firstparam);
					else ch.writeInfo(fromnick+" " +Irc.language.get("join")+ " " + firstparam);
					if (!fromnick.equals(UIHandler.nick)) {
						ch.addNick(Window.MODE_NONE, fromnick);
					}
				}
				else if (command[0].equals("PART")) {
					if (!fromnick.equals(UIHandler.nick)) {
						if (command.length == 1) {
							ch = uihandler.getChannel(cmdline[2]);
							if(showhosts)ch.writeInfo(fromnick +" ("+cmdline[0].substring(cmdline[0].indexOf('!')+1)+") " +Irc.language.get("part")+ " " + cmdline[2]);
							else ch.writeInfo(fromnick +" " +Irc.language.get("part")+ " " + cmdline[2]);
						}
						else {
							ch = uihandler.getChannel(command[1]);
							ch.writeInfo(fromnick + " "+Irc.language.get("part")+" " + command[1] + " (" + cmdline[2] + ")");
						}
						ch.deleteNick(fromnick);
					}
				}
				else if (command[0].equals("KICK")) {
					if (command[2].equals(UIHandler.nick)) { // you got kicked
						uihandler.getConsole().writeInfo(Irc.language.get("youkicked")+"  "+ fromnick + " (" + cmdline[2] + ")");
						if(ajoinkick){
							uihandler.getChannel(command[1]).deleteAllNicks();
							Irc.writeLine("JOIN "+command[1]);
						}else uihandler.getChannel(command[1]).close();
					}
					else {
						ch = uihandler.getChannel(command[1]);
						ch.writeInfo(fromnick + " "+Irc.language.get("kick")+" " + command[2] + " (" + cmdline[2] + ")");
						ch.deleteNick(command[2]);
						
					}
				}
				else if (command[0].equals("TOPIC")) {
					ch = uihandler.getChannel(command[1]);
					ch.writeInfo(fromnick + " "+Irc.language.get("changetopic")+" '" + cmdline[2] + "'");
					ch.topic = cmdline[2];
				}
			}
			else {
				switch(cmdnum) {
					case 001:  // RPL_WELCOME
						uihandler.getConsole().writeInfo(Irc.language.get("connected"));
						String[] start_commands = this.start_commands;
						if (start_commands !=null) {
							for (int i=0; i<start_commands.length; i++) {
								uihandler.getConsole().handleMsg(start_commands[i]);
							}
						}
						String[] channels = this.channels;
						if (channels !=null) {
							for (int i=0; i<channels.length; i++) {
								Irc.writeLine("JOIN " + channels[i]);
							}
						}
						needupdate = true;
						break;
					case 301:  // RPL_AWAY
						addWhois(command[2].toUpperCase(), Irc.language.get("awaystatus")+": "+ cmdline[2] + "\n");
//						uihandler.getConsole().writeAction("* " + command[2] + " "+Irc.language.get("away")+": " + cmdline[2]);
						break;
					case 305:  // RPL_UNAWAY
						addWhois(command[2].toUpperCase(), Irc.language.get("awaystatus")+": "+ cmdline[2] + "\n");
//						uihandler.getConsole().writeAction("* " + cmdline[2]);
						break;
					case 306:  // RPL_NOWAWAY
						addWhois(command[2].toUpperCase(), Irc.language.get("awaystatus")+": "+ cmdline[2] + "\n");
//						uihandler.getConsole().writeAction("* " + cmdline[2]);
						break;
					case 311:  // RPL_WHOISUSER
						String string = Irc.language.get("nick")+": " + command[2] + "\n";
						string += Irc.language.get("name")+": " + command[3].substring(1) + "\n";
						string += Irc.language.get("host")+": " + command[4] + "\n";
						addWhois(command[2].toUpperCase(), string);
						break;
					case 312:  // RPL_WHOISSERVER
						addWhois(command[2].toUpperCase(), Irc.language.get("whoisoperator")+": "+cmdline[2] + "\n");
						break;
					case 307:  // RPL_WHOISREGISTERED
						addWhois(command[2].toUpperCase(), command[2]+": " + cmdline[2] + "\n");
						break;
					case 313:  // RPL_WHOISOPERATOR
						addWhois(command[2].toUpperCase(), Irc.language.get("whoisoperator")+": " + cmdline[2] + "\n");
						break;
					case 314:  // RPL_WHOWASUSER
						addWhois(command[2].toUpperCase(), Irc.language.get("realname")+": " + command[5] + " \n");
						break;
					case 317:  // RPL_WHOISIDLE
						addWhois(command[2].toUpperCase(), Irc.language.get("idle")+": " + parseTime(command[3]) + " \n");
						break;
					case 319:  // RPL_WHOISCHANNELS
						System.out.println("="+cmdnum+"="+command[2]+"="+cmdline[2]);
						addWhois(command[2], Irc.language.get("whoischannels")+": " + cmdline[2] + "\n");
						break;
					case 318:  // RPL_ENDOFWHOIS
						mb = new MessageBox(display, uihandler.getCurrentWindow(), null, null, uihandler.getCurrentWindow().cmd_closenamfav, "WHOIS", whois, MessageBox.TYPE_INFO, Font.SIZE_SMALL, 0);
						uihandler.setDisplay(mb);
						whois = "";
						//whois.clear();
						break;
					case 369:  // RPL_ENDOFWHOWAS
					case 321:  // RPL_LISTSTART
					case 322:  // RPL_LIST
					case 323:  // RPL_LISTEND
						ch = uihandler.getChannel(Irc.language.get("channelslist"));
						String str = cmdline[1].substring(cmdline[1].indexOf("#")); 
						ch.writeAction(str.substring(0, str.indexOf(" "))+"["+str.substring(str.indexOf(" ")+1, str.length()-1)+"]");
						ch.addNick(Window.MODE_NONE, str.substring(0, str.indexOf(" ")));
						ch.writeInfo(cmdline[2]);
					case 324:  // RPL_CHANNELMODEIS
						break;
					case 331:  // RPL_NOTOPIC
						uihandler.getChannel(command[2]).writeInfo(Irc.language.get("notopic"));
						break;
					case 332:  // RPL_TOPIC
						uihandler.getChannel(command[2]).writeInfo(Irc.language.get("topic")+": '" + cmdline[2] + "'");
						uihandler.getChannel(command[2]).topic = cmdline[2];
						break;
					case 333:  // RPL_TOPICWHOTIME
						String topicwho = "";
						topicwho += Irc.language.get("topicwho")+" '" + command[3].substring(0, command[3].indexOf('!')) + "'";
						//topicwho += " \u043D\u0430 " + Utils.formatDateMillis(Long.parseLong(cmdline[2]));
						uihandler.getChannel(command[2]).writeInfo(topicwho);
						// XXX this has to be fixed, time formatting....
						break;
					case 341:  // RPL_INVITING
						uihandler.getChannel(command[2]).writeAction(Irc.language.get("inviteonchannel")+" " + command[3]);
						break;
					case 342:  // RPL_SUMMONING
					case 351:  // RPL_VERSION
						uihandler.getConsole().writeInfo(cmdline[2]);
					case 352:  // RPL_WHOREPLY
					case 315:  // RPL_ENDOFWHO
						// FIXME: not implemented
						break;
					case 353:  // RPL_NAMREPLY
						ch = uihandler.getChannel(command[3]);
						String[] s = Utils.splitString(cmdline[2].trim(), " ");
						for (int i=0; i<s.length; i++) {
							String ret;
							char mode = Window.MODE_NONE;
							if (s[i].charAt(0) == '@') mode = Window.MODE_OP;
							else if (s[i].charAt(0) == '%') mode = Window.MODE_HALFOP;
							else if (s[i].charAt(0) == '+') mode = Window.MODE_VOICE;

							if (mode != Window.MODE_NONE) ret = s[i].substring(1);
							else ret = s[i];

							ch.addNick(mode, ret);
						}
						break;
					case 366:  // RPL_ENDOFNAMES
						uihandler.getChannel(command[2]).printNicks();
						break;
					case 367:  // RPL_BANLIST
					case 368:  // RPL_ENDOFBANLIST
						// FIXME: no ban implemented
						uihandler.getConsole().writeInfo("Ban = "+command[3]);
						break;
					case 371:  // RPL_INFO
					case 374:  // RPL_ENDOFINFO
						uihandler.getConsole().writeInfo(cmdline[2]);
						break;
					case 375:  // RPL_MOTDSTART
					case 372:  // RPL_MOTD
						if (showinput) uihandler.getConsole().writeAction(cmdline[2]);
						break;
					case 376:  // RPL_ENDOFMOTD
						// skip message of the day
						break;
					case 381:  // RPL_YOUREOPER
					case 382:  // RPL_REHASHING
					case 391:  // RPL_TIME
					case 392:  // RPL_USERSSTART
					case 393:  // RPL_USERS
					case 394:  // RPL_ENDOFUSERS
					case 395:  // RPL_NOUSERS
						// FIXME: not implemented, most can be just handled elsewhere
						break;

					case 431:  // ERR_NONICKNAMEGIVEN
					case 432:  // ERR_ERRONEUSNICKNAME
					case 433:  // ERR_NICKNAMEINUSE
						if (!nicktried && !altnick.trim().equals("")) {
							uihandler.getConsole().writeInfo(Irc.language.get("nickinuse")+" '" + altnick + "\'");
							Irc.writeLine("NICK " + altnick);
							UIHandler.nick = altnick;
							nicktried = true;
						}
						else {
							Window co = uihandler.getConsole();
							co.nickChangeAction();
						}
						break;
					case 471:  // ERR_CHANNELISFULL
					case 473:  // ERR_INVITEONLYCHAN
					case 474:  // ERR_BANNEDFROMCHAN
					case 475:  // ERR_BADCHANNELKEY
						uihandler.getConsole().writeInfo(cmdline[2] + " "+Irc.language.get("badchannelkey")+" " + command[2]);
						break;
					default:
						if (showinput)
							uihandler.getConsole().writeAction("-" + response);
						break;
				}
			}
		}
		catch (Exception e) { e.printStackTrace(); }
	}
	
	public void setNeedUpdate(boolean value) {
		needupdate = value;
	}

	private void addWhois(String nick, String str) {
		uihandler.getConsole().writeAction("* "+nick+" "+str);
		whois += str;
	}

	private String[] parseLine(String input) {
		int idx1, idx2;
		String[] ret = new String[3];
		
		if (input.charAt(0) == ':') {
			ret[0] = input.substring(1, input.indexOf(' '));
			idx1 = input.indexOf(" ")+1;
		}
		else {
			ret[0] = null;
			idx1 = 0;
		}

		idx2 = input.indexOf(":", idx1);
		if (idx2 != -1) {
			ret[1] = input.substring(idx1, idx2);
			ret[2] = input.substring(idx2+1);
		}
		else if ((idx2 = input.lastIndexOf(' ')) != -1) {
			ret[1] = input.substring(idx1, idx2);
			ret[2] = input.substring(idx2+1);
		}
		else {
			ret[1] = input.substring(idx1);
			ret[2] = null;
		}

		return ret;
	}

	private String parseTime(String seconds) {
		int sec, min, hour, day;
		String ret = "";

		try {
			sec = Integer.parseInt(seconds);
			day = sec/86400;
			sec -= day*86400;
			hour = sec/3600;
			sec -= hour*3600;
			min = sec/60;
			sec -= min*60;
			ret = new String(day + Irc.language.get("day")+" " + hour + Irc.language.get("hour")+" " + min + Irc.language.get("min")+" " + sec + Irc.language.get("sec"));
		} catch (NumberFormatException nfe) { ; }
		return ret;
	}
}
