package inettools;

import java.io.*;

import javax.microedition.io.*;

import lib.Utils.Utils;

public class SocketIrc extends IrcConnection {
	private StreamConnection connector;
	private DataInputStream in;
	private DataOutputStream out;
	private String encoding, outbuf;
	private boolean connected;

	private int bytein;
	private int byteout;

	public SocketIrc(String charset) {
		encoding = charset;

		bytein = 0;
		byteout = 0;
		connected = false;
	}

	public String connect(String host, int port, String init) {
		String ret = null;

		try {
			connector = (StreamConnection) Connector.open("socket://" + host + ":" + port, Connector.READ_WRITE);
			in = connector.openDataInputStream();
			out = connector.openDataOutputStream(); 

			outbuf = null;
			connected = true;
			ret = writeData(init);

		} catch (Exception e) {
			ret = Irc.language.get("errorconnecting");
			ret += Irc.language.get("exception")+": " + e.getMessage();
			return ret;
		}
		return null;
	}

	public void disconnect() {
		if (connected) {
		      //   .
		      if (in != null)
		      	try {in.close();} catch (IOException ex){}
		      if (out != null)
		      	try {out.close();} catch (IOException ex){}
		      if (connector != null)
		      	try {connector.close();} catch (IOException ex){}
		      in = null;
		      out = null;
		      connector = null;
              connected = false;
		}
	}
	
	public String updateConnection() {
		byte[] tmp;
		String ret;

		if (outbuf != null && connected) {
			try {
				tmp = stringToByteArray(outbuf, encoding);
				outbuf = null;
				out.write(tmp);
				out.flush();
				byteout += tmp.length;
			}
			catch (Exception e) {
				ret = Irc.language.get("socketioerror");
				ret += Irc.language.get("Exception")+": " + e.getMessage();
				connected = false;
				return ret;
			}
		}
		return null;
	}

	public String readLine() {
		int ch;
		String ret;
		byte[] buf;

		ret = null;
		try {
			buf = Utils.readLine(in);
			if (buf==null) return null;
			bytein += buf.length + 40;
			ret = byteArrayToString(buf, encoding);

		} catch(Exception e) {
			connected = false;
		}
		return ret;
	}
	
	public String writeData(String input) {
		if (outbuf == null) outbuf = input;
		else outbuf += input;

		return updateConnection();
	}

	public boolean hasDataInBuffer() {
		try {
			return in.available()>0;
		} catch (IOException e) {
			return connected;
		}
	}

	public boolean isConnected() {
		return connected;
	}

	public int getBytesIn() {
		return bytein;
	}

	public int getBytesOut() {
		return byteout;
	}
}
