package inettools;

import javax.microedition.lcdui.Font;

import lib.GraphicControls.TextList;

public class TextArea extends TextList{
	private UIHandler uihandler;
	private int buffer;

	public TextArea(int textListWidth, int textListHeight, int fontSize, int cursorStyle, int buffer, UIHandler uihandler) {
		super(textListWidth, textListHeight, fontSize, cursorStyle, uihandler.emotions.smilesList);
		this.buffer = buffer;
		this.uihandler = uihandler;
	}
//	======================================
	public String getSelectedMessage(){
		return getItemText(currentItem);
	}
//	======================================
	public void addIRCText(String text, int itemIndex, boolean smiles){
		addIRCText(text, 0x000000, 0xffffff, Font.STYLE_PLAIN, fontSize, itemIndex, smiles);
	}
	public void addIRCText(String text, boolean smiles){
		addIRCText(text, 0x000000, 0xffffff, Font.STYLE_PLAIN, fontSize, items.size(), smiles);
	}
	public void addIRCText(String text, int fontColor, int bgColor, int fontStyle, int fontSize, int textIndex, boolean smiles){
		int[] mIRCColors = {15, 0, 4, 2, 9, 1, 5, 3, 11, 10, 6, 14, 12, 13, 8, 7};

		for (int i=0; i<text.length(); i++) {
			char ch = text.charAt(i);
			if (ch == 2 | ch == 3 | ch == 22 | ch == 31) {
				if (!text.substring(0, i).equals("")){
					if(smiles) uihandler.emotions.addTextWithSmiles(this, text.substring(0, i), fontColor, bgColor, fontStyle, fontSize, textIndex);
					else addTextWithURL(text.substring(0, i), fontColor, bgColor, fontStyle, fontSize, textIndex);
				}

				text = text.substring(i+1);
				i = -1;

				switch (ch) {
					case 2:  // BOLD
						fontStyle ^= Font.STYLE_BOLD;
						break;
					case 31: //UNDERLINE
						fontStyle ^= Font.STYLE_UNDERLINED;
						break;
					case 22: // ITALIC
						fontStyle ^= Font.STYLE_ITALIC;
						break;
					case 3:  // mIRC COLOUR
						String[] cols = new String[2];
						for (int j=0; j<text.length(); j++) {
							char c = text.charAt(j);
							if (c != ',' && !Character.isDigit(c)) {
								if (j==0) break;
								if (cols[0] == null)
									cols[0] = text.substring(0, j);
								else cols[1] = text.substring(0, j);
								text = text.substring(j);
								break;
							}
							else if (c == ',' && cols[0] != null) {
								if (j==0) break;
								cols[1] = text.substring(0, j);
								text = text.substring(j);
								break;
							}
							else if (c == ',') {
								if (j==0) {
									text.substring(j+1);
									break;
								}
								cols[0] = text.substring(0, j);

								if (j == text.length()-1 || !Character.isDigit(text.charAt(j+1))) {
									text = text.substring(j);
									break;
								}
								else {
									text = text.substring(j+1);
									j=0;
								}
							}
						}
						if (cols[0] == null) {
							fontColor = 0x000000;
							bgColor   = 0xffffff;
						}
						else {
							if (cols[0] != null) {
								fontColor = getColor(mIRCColors[Integer.parseInt(cols[0])<16 ? Integer.parseInt(cols[0]) : 2]);
							}
							if (cols[1] != null) {
								bgColor = getColor(mIRCColors[Integer.parseInt(cols[1])<16 ? Integer.parseInt(cols[1]) : 2]);
							}
						}
						break;
				}
			}
		}
		if (!text.equals("")) {
			if(smiles)uihandler.emotions.addTextWithSmiles(this, text, fontColor, bgColor, fontStyle, fontSize, textIndex);
			else addTextWithURL(text, fontColor, bgColor, fontStyle, fontSize, textIndex);
		}
	}
	
	public void buffer(){
		if(items.size() > buffer && topItem > 0 && currentItem > 0){
			items.removeElementAt(0);
			topItem--;
			currentItem--;
		}
	}

    private int getColor(int numb) {
		switch (numb) {
			case 0:  return 0x00000000;
			case 1:  return 0x00aa0000;
			case 2:  return 0x0000d200;
			case 3:  return 0x00aa5522;
			case 4:  return 0x000000aa;
			case 5:  return 0x00aa00aa;
			case 6:  return 0x0000aaaa;
			case 7:  return 0x00aaaaaa;
			case 8:  return 0x00444444;
			case 9:  return 0x00ff4444;
			case 10:  return 0x0044ff44;
			case 11:  return 0x00ffff44;
			case 12:  return 0x004444ff;
			case 13:  return 0x00ff44ff;
			case 14:  return 0x0044ffff;
			case 15:  return 0x00ffffff;
		}
		return 0x00FFFFFF;
	}
	public void clear(){
		items.removeAllElements();
		topPosition = 0;
		topItem = 0;
		currentItem = 0;
	}
}
