package lib.Font;
import java.io.InputStream;

import javax.microedition.lcdui.Graphics;

public class CopyOfSampleFont {
	public static final int 	STYLE_BOLD = 0,
								STYLE_PLAIN = 1,
								STYLE_UNDERLINED = 2;
	private String[][] fontData;
	private InputStream in;
	public int 	size,
				style 		= STYLE_PLAIN,
				bgColor 	= 0xffffff,
				fontColor 	= 0x000000,
				height,
				width[];
	
	public CopyOfSampleFont(String fontFileName) {
		in = getClass().getResourceAsStream(fontFileName);
		try {
			size = in.read()+1;
			height = in.read();
			width = new int[size];
			for(int i=0; i<size; i++){
				width[i] = in.read();
			}
			fontData = new String[size][height];
			for(int i=0; i<size; i++){
				for(int j=0; j<height; j++){
					fontData[i][j]="";
					for(int k=0; k<bytesPerSymbol(width[i]); k++){
						fontData[i][j]+=DecToBin(in.read());
					}
				}
			}
		} catch (Exception e) {}
		
	}
	
	private String DecToBin(int input){
		String result=Integer.toBinaryString(input);
		
		if(result.length()<8){
			result="00000000".substring(result.length())+result;
		}
		
		return result;
	}

    public void setBgColor(int red, int green, int blue){
        if(red < 0 || red > 255 || green < 0 || green > 255 || blue < 0 || blue > 255){
            throw new IllegalArgumentException("Value out of range");
        }else{
        	bgColor = red << 16 | green << 8 | blue;
            return;
        }
    }
    public void setBgColor(int RGB){
        int red = RGB >> 16 & 0xff;
        int green = RGB >> 8 & 0xff;
        int blue = RGB & 0xff;
        bgColor = RGB & 0xffffff;
    }
    public void setFontColor(int red, int green, int blue){
        if(red < 0 || red > 255 || green < 0 || green > 255 || blue < 0 || blue > 255){
            throw new IllegalArgumentException("Value out of range");
        } else{
        	fontColor = red << 16 | green << 8 | blue;
            return;
        }
    }
    public void setFontColor(int RGB)
    {
        int red = RGB >> 16 & 0xff;
        int green = RGB >> 8 & 0xff;
        int blue = RGB & 0xff;
        fontColor = RGB & 0xffffff;
    }
	public void setStyle(int style){
        if(style < STYLE_BOLD || style > STYLE_UNDERLINED){
            throw new IllegalArgumentException("Value out of range");
        }else this.style = style; 
	}
//	===========================
    private int getStyle(){
    	return style;
    }
    private int getBgColor(){
    	return bgColor;
    }
    private int getFontColor(){
    	return fontColor;
    }
//	===========================
    public int substringWidth(String str, int index1){
    	int result = 0;
		for(int i=index1; i<str.length(); i++){
			int ch = str.charAt(i);
			if(ch>=1040 && ch<=1103) ch-=848;
			result+=width[ch];
		}
    	return result;    	
    }
    public int substringWidth(String str, int index1, int index2){
    	int result = 0;
		for(int i=index1; i<index2; i++){
			int ch = str.charAt(i);
			if(ch>=1040 && ch<=1103) ch-=848;
			result+=width[ch];
		}
    	return result;    	
    }
    public int stringWidth(String str){
    	int result = 0;
		for(int i=0; i<str.length(); i++){
			int ch = str.charAt(i);
			if(ch>=1040 && ch<=1103) ch-=848;
			result+=width[ch];
		}
    	return result;    	
    }

	public void drawString(Graphics g, String text, int left, int top){
		for(int i=0; i<text.length(); i++){
			int ch = text.charAt(i);
			if(ch>=1040 && ch<=1103) ch-=848;
			drawChar(g, ch, left, top);
			left+=width[ch];
		}
		
	}
	public void drawChar(Graphics g, int ch, int left, int top){
		int x=left,
			bold=0;
		
		if(style==STYLE_BOLD ||style==(STYLE_BOLD | STYLE_UNDERLINED)) bold=1;
		if(bgColor!=0xffffff){
			g.setColor(bgColor);
			g.fillRect(left, top, width[ch]+bold, height);
		}
		g.setColor(fontColor);
		if(style==STYLE_UNDERLINED ||style==(STYLE_BOLD | STYLE_UNDERLINED)) g.drawLine(x, top+height, x+width[ch]+bold, top+height);

		for(int i=0; i<height; i++){
			for(int j=0; j<width[ch] && j<fontData[ch][i].length(); j++){
				if(fontData[ch][i].charAt(j)=='0') g.drawLine(x, top, x+bold, top);
				x++;
			}
			top++;
			x=left;
		}
	}
	private int bytesPerSymbol(int width){
		if(width<=8) return 1;
		else if(width - width/8*8 == 0) return width/8;
		else return width/8+1;
	}

}

