package lib.GraphicControls;

import javax.microedition.lcdui.AlertType;
import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import lib.Font.SampleFont;
import lib.Utils.Utils;

public class MessageBox extends Canvas{
	public static final int TYPE_ALARM = 0;
	public static final int TYPE_INFO = 1;
	public static final int TYPE_CONFIRMATION = 2;
	public static final int TYPE_ERROR = 3;
	public static final int TYPE_WARNING = 4;

	private Image ti;
	private Graphics tg;
	public TextList textlist;
	private int width,
				height,
				left,
				top,
				fontSize,
				type,
				heightdiff,
				fontHeight;
	private String 	title = null,
					text = null;
	private Command cmd_left,
					cmd_center,
					cmd_right;
	private CommandListener cl;
	private Display display;
	private Displayable displayable;
	
	private Font font = null;
	private SampleFont sampleFont = null;
	
	private runningTitle rt;

	private class runningTitle extends Thread{
		public boolean run = true;
		private String temp;
		private int size;

		public runningTitle(){
			temp = title;
			size = (width-20)/font.charWidth('D');
			title = "";
		}
		public void run() {
			while(run){
				title = temp.substring(0, size);
				temp = temp.substring(1) + temp.charAt(1);
				repaint(left + 10, top + 5, width - 25, fontHeight + 5);
				try {sleep(500);}catch (InterruptedException e){}
			}
		}
	}

	public MessageBox(Display display, CommandListener cl, Command cmd_left, Command cmd_center, Command cmd_right, String title, String text, int type, int fontSize, int heightdiff){
		//FIXME
		//MIDP 2.0
		//setFullScreenMode(true);
		
		this.display = display;
		this.cl = cl;
		this.cmd_left = cmd_left;
		this.cmd_center = cmd_center;
		this.cmd_right = cmd_right;
		this.title = title;
		this.text = text;
		this.fontSize = fontSize;
		this.heightdiff = heightdiff;
		
		width = getWidth();
		height = getHeight()+heightdiff;
		
		if(height>120){
			left = 4;
			top = 16;
		}else{
			left = 0;
			top = 0;
		}
		width  = width  - 2*left;
		height = height - 2*top;
		
		if(height > width){
			top = height - width;
			height = width -10;
		}
		
		font = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, 8);
		fontHeight = font.getHeight();

		textlist= new TextList(width - 10, height - 2*fontHeight - 14, fontSize, TextList.CURSOR_NONE, null);
		textlist.addText(text);
		
		if(font.stringWidth(title)> width -12) {
			rt = new runningTitle();
			rt.setPriority(Thread.MIN_PRIORITY);
			rt.start();
		}
	}
	public MessageBox(Display display, Displayable displayable, String title, String text, int type, int fontSize, int heightdiff){
		this.display = display;
		this.displayable = displayable;
		this.title = title;
		this.text = text;
		this.fontSize = fontSize;
		this.heightdiff = heightdiff;
		
		width = getWidth();
		height = getHeight()+heightdiff;
		
		left = 4;
		top = 16;
		width  = width  - 2*left;
		height = height - 2*top;
		
		if(height > width){
			top = height - width;
			height = width -10;
		}
		
		if(fontSize<0){
			sampleFont = new SampleFont("/fonts/font.bin");
			fontHeight = sampleFont.height;
			if(sampleFont.stringWidth(title)> width -12) {
				rt = new runningTitle();
				rt.setPriority(Thread.MIN_PRIORITY);
				rt.start();
			}
		}else{
			font = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, fontSize);
			fontHeight = font.getHeight();
			if(font.stringWidth(title)> width -12) {
				rt = new runningTitle();
				rt.setPriority(Thread.MIN_PRIORITY);
				rt.start();
			}
		}

		textlist= new TextList(width - 10, height - 2*fontHeight - 14, fontSize, TextList.CURSOR_NONE, null);
		textlist.addText(text);
	}
//	=============================================
	public void playSound(){
		if(type == TYPE_ALARM){
			AlertType.ALARM.playSound(display);
		}else if(type == TYPE_INFO){
			AlertType.INFO.playSound(display);
		}else if(type == TYPE_CONFIRMATION){
			AlertType.CONFIRMATION.playSound(display);
		}else if(type == TYPE_ERROR){
			AlertType.ERROR.playSound(display);
		}else if(type == TYPE_WARNING){
			AlertType.WARNING.playSound(display);
		}
	}
	public void setDisplayable(Displayable displayable){
		this.displayable = displayable;
	}
	public void show(){
		display.setCurrent(this);
		playSound();
	}
//	=============================================	
	public void setText(String text){
		this.text = text;
		textlist.clear();
		textlist.addText(text);
	}
	public void setTitle(String title){
		this.title = title;
	}
//	=============================================	
	public String getText(){
		return text;
	}
	public String getTitle(){
		return title;
	}
//	=============================================	
	protected void paint(Graphics g) {
		g.setColor(50, 50, 50);
		g.drawRect(left , top, width-1, height-1);
		g.setColor(191, 193, 208);
		g.drawRect(left + 1, top + 1, width-3, height-3);
		g.setColor(241, 241, 241);
		g.drawRect(left + 1, top + 1, width-4, height-4);
		g.setColor(219, 222, 232);
		g.drawRect(left + 2, top + 2, width-5, height-5);
		g.setColor(255, 255, 255);
		g.drawRect(left + 2, top + 2, width-6, height-6);
		g.setColor(212, 208, 200);
		g.drawRect(left + 3, top + 3,width-7, height-7);
		g.setColor(241, 243, 248);
		g.fillRect(left + 4, top +4, width-8, height-8);

		if(sampleFont!=null) Utils.drawFramedText(g, sampleFont, title, left + width/2 - sampleFont.stringWidth(title)/2, top+5, 0xffffff, 0x000000);
		else  Utils.drawFramedText(g, font, title, left + width/2 - font.stringWidth(title)/2, top+5, 0xffffff, 0x000000);

		if(cmd_left != null){
			if(sampleFont!=null) Utils.drawFramedText(g, sampleFont, "*", left+10, top+height-fontHeight-6, 0x00ff00, 0x000000);
			else  Utils.drawFramedText(g, font, "*", left + 10, top+height-fontHeight-6, 0x00ff00, 0x000000);
		}
		if(cmd_center != null){
			if(sampleFont!=null) Utils.drawFramedText(g, sampleFont, "+", left + width/2 -2, top+height-fontHeight-6, 0x0000ff, 0x000000);
			else  Utils.drawFramedText(g, font, "+", left + width/2 -2, top+height-fontHeight-6, 0x0000ff, 0x000000);
		}
		if(cmd_right != null || displayable != null){
			if(sampleFont!=null) Utils.drawFramedText(g, sampleFont, "#", left+width-15, top+height-fontHeight-6, 0xff0000, 0x000000);
			else  Utils.drawFramedText(g, font, "#", left+width-15, top+height-fontHeight-6, 0xff0000, 0x000000);
		}
		
		textlist.paint(g, left + 5, top + fontHeight + 7);
	}
	protected void keyPressed(int keyCode){
		if(getGameAction(keyCode) == UP	|| keyCode == KEY_NUM2 || keyCode == -13){
			textlist.up();
			repaint(left + 5, top + fontHeight + 7, width - 8, height - 2*fontHeight - 12);
		}else if(getGameAction(keyCode) == DOWN	|| keyCode == KEY_NUM8 || keyCode == -14){
			textlist.down();
			repaint(left + 5, top + fontHeight + 7, width - 8, height - 2*fontHeight - 12);
		}else if(keyCode == KEY_NUM9){
			textlist.pageDown();
			repaint(left + 5, top + fontHeight + 7, width - 8, height - 2*fontHeight - 12);
		}else if(keyCode == KEY_NUM3){
			textlist.pageUp();
			repaint(left + 5, top + fontHeight + 7, width - 8, height - 2*fontHeight - 12);
		}else if(keyCode == KEY_NUM1){
			textlist.home();
			repaint(left + 5, top + fontHeight + 7, width - 8, height - 2*fontHeight - 12);
		}else if(keyCode == KEY_NUM7){
			textlist.end();
			repaint(left + 5, top + fontHeight + 7, width - 8, height - 2*fontHeight - 12);
		}
		
		if(getGameAction(keyCode) == FIRE	|| keyCode == KEY_NUM0 ){
			if(cmd_center != null)cl.commandAction(cmd_center, null);
		}
		if(keyCode == KEY_STAR || keyCode ==-6 || keyCode ==-1 || keyCode ==-11){
			if(cmd_left != null)cl.commandAction(cmd_left, null);
		}
		if(keyCode == -4 || keyCode ==-7 || keyCode ==KEY_POUND || keyCode ==-12){
			if(cmd_right != null)cl.commandAction(cmd_right, null);
			else if(displayable != null)display.setCurrent(displayable);
		}	
	}
	
}
