 package lib.GraphicControls;

import java.util.Vector;

import javax.microedition.lcdui.Font;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

import lib.Font.SampleFont;
import lib.Resources.ImageList;
import lib.Utils.Utils;

public class TextList{
	final public static int CURSOR_NONE = 0,
							CURSOR_RECT = 1,
							
							TYPE_PLAIN  = 0,
							TYPE_URL    = 1;
	public int  textListWidth, 
				textListHeight,
				fontSize,
				cursorStyle = CURSOR_NONE,
				currentItem = 0,
				topItem = 0,
				topPosition = 0;
	public Vector items;
	private ImageList images;
	private SampleFont sampleFont = null;
	
//	=========ITEM=======================	
	private class Item{
		public Image image = null;
		public String text = null;
		public int 	fontColor, 
					bgColor,
					fontStyle,
					fontSize,
					width,
					height,
					imageIndex = -1;
		private SampleFont sampleFont = null;
		
		public Item(Image image){
			this.image = image;
			
			width  = image.getWidth();
			height = image.getHeight(); 
		}
		public Item(int imageIndex){
			this.imageIndex = imageIndex;
			
			width  = images.getWidth();
			height = images.getHeight(); 
		}
		public Item(String text, int fontColor, int bgColor, int fontStyle, int fontSize){
			this.text 			= text;
			this.fontColor 		= fontColor;
			this.bgColor 		= bgColor;
			this.fontStyle		= fontStyle;
			this.fontSize		= fontSize;
			
			Font font = Font.getFont(Font.FACE_SYSTEM, fontStyle, fontSize);
			width  = font.stringWidth(text);
			height = font.getHeight();
			font = null;
		}
		public Item(String text, SampleFont sampleFont){
			this.text 			= text;
			this.sampleFont		= sampleFont;

			width  = sampleFont.stringWidth(text);
			height = sampleFont.height;
		}

		public void paint(Graphics g, int left, int top){
			if(image != null){
				g.drawImage(image, left, top + height-image.getHeight(), Graphics.LEFT | Graphics.TOP);
				left += image.getWidth();
			}else if(images!=null && imageIndex>-1 && imageIndex<images.getSize()){
				images.drawImage(g, imageIndex, left, top + height-images.getHeight());
				left += images.getWidth();
			}else if(text != null){
				if(sampleFont!=null){
					sampleFont.drawString(g, text, left, top + height - sampleFont.height);
				}else{
					Font font = Font.getFont(Font.FACE_SYSTEM, fontStyle, fontSize);
					g.setFont(font);
					if(bgColor != 0xffffff){
						g.setColor(bgColor);
						g.fillRect(left, top + height - font.getHeight(), font.stringWidth(text), font.getHeight());
					}
					g.setColor(fontColor);
					g.drawString(text, left, top + height - font.getHeight(), Graphics.LEFT | Graphics.TOP);
					font = null;
				}
			}
		}
	}
//	=========LINE ITEM=======================	
	private class LineItem{
		private Vector items;
		public int 	width = 0,
					height = 0;
		
		public LineItem(){
			items = new Vector();
		}
		
		public boolean addImage(Image image){
			if(width + image.getWidth() < textListWidth - 5){
				width += image.getWidth();
				items.addElement(new Item(image));
				if(height < image.getHeight()){
					height = image.getHeight();
					for(int i=0; i<items.size(); i++)
						if(((Item) items.elementAt(i)).height < height)
							((Item) items.elementAt(i)).height = height;
				}else ((Item) items.lastElement()).height = height;
				return true;
			}else return false;
		}
		public boolean addImage(int imageIndex){
			if(width + images.getWidth() < textListWidth - 5){
				width += images.getWidth();
				items.addElement(new Item(imageIndex));
				if(height < images.getHeight()){
					height = images.getHeight();
					for(int i=0; i<items.size(); i++)
						if(((Item) items.elementAt(i)).height < height)
							((Item) items.elementAt(i)).height = height;
				}else ((Item) items.lastElement()).height = height;
				return true;
			}else return false;
		}
		public boolean addText(String text, int fontColor, int bgColor, int fontStyle, int fontSize){
			Font font = Font.getFont(Font.FACE_SYSTEM, fontStyle, fontSize);
			if(width + font.stringWidth(text) < textListWidth - 5){
				width += font.stringWidth(text);
				items.addElement(new Item(text, fontColor, bgColor, fontStyle, fontSize));
				if(height < font.getHeight()){
					height = font.getHeight();
					for(int i=0; i<items.size(); i++)
						if(((Item) items.elementAt(i)).height<height)((Item) items.elementAt(i)).height=height;
				}else ((Item) items.lastElement()).height=height;
				font = null;
				return true;
			}else{
				font = null;
				return false;
			}
		}
		public boolean addText(String text, SampleFont sampleFont){
			if(width + sampleFont.stringWidth(text) < textListWidth - 5){
				width += sampleFont.stringWidth(text);
				items.addElement(new Item(text, sampleFont));
				if(height < sampleFont.height){
					height = sampleFont.height;
					for(int i=0; i<items.size(); i++)
						if(((Item) items.elementAt(i)).height<height)((Item) items.elementAt(i)).height=height;
				}else ((Item) items.lastElement()).height=height;
				return true;
			}else{
				return false;
			}
		}
		public void paint(Graphics g, int left, int top){
			if(!items.isEmpty())
				for(int i=0; i<items.size(); i++){
					((Item) items.elementAt(i)).paint(g, left, top);
					left+=((Item) items.elementAt(i)).width;
				}
		}
	}
//	=========LIST ITEM=======================	
	private class ListItem{
		private Vector lines;
		public String text = "";
		public int 	height = 0,
					type = TYPE_PLAIN;
		
		public ListItem(){
			lines = new Vector();
		}
		
		public void addImage(Image image){
			if(lines.isEmpty()) lines.addElement(new LineItem());
			
			if(!((LineItem) lines.lastElement()).addImage(image)){
				lines.addElement(new LineItem());
				((LineItem) lines.lastElement()).addImage(image);
			}
			getHeight();
		}
		public void addImage(int imageIndex){
			if(lines.isEmpty()) lines.addElement(new LineItem());
			
			if(!((LineItem) lines.lastElement()).addImage(imageIndex)){
				lines.addElement(new LineItem());
				((LineItem) lines.lastElement()).addImage(imageIndex);
			}
			getHeight();
		}
		public void addText(String text, int fontColor, int bgColor, int fontStyle, int fontSize){
			String[] strings, words;
			
			this.text += text;
			if(lines.isEmpty()) lines.addElement(new LineItem());

			text = Utils.replace(text, "\r\n", "\n\n");
			text = Utils.replace(text, "\r", "\n");
			
			strings = Utils.splitString(text, "\n");
			for(int i=0; i<strings.length; i++){
				if(strings[i].equals("")) {
					//lines.addElement(new LineItem());
					//((LineItem) lines.lastElement()).addText(" ", fontColor, bgColor, fontStyle, fontSize);
				}else{
					if(strings.length > 1)lines.addElement(new LineItem());
					words = Utils.splitString(strings[i], " ");
					for(int j=0; j<words.length; j++){
						if(!((LineItem) lines.lastElement()).addText(words[j]+" ", fontColor, bgColor, fontStyle, fontSize)){
							lines.addElement(new LineItem());
							if(words[j].length() == 0)((LineItem) lines.lastElement()).addText(" ", fontColor, bgColor, fontStyle, fontSize);
							else if(!((LineItem) lines.lastElement()).addText(words[j]+" ", fontColor, bgColor, fontStyle, fontSize)){
								String[] word = Utils.splitString(words[j], Font.getFont(Font.FACE_SYSTEM, fontStyle, fontSize), textListWidth-10);
								for(int k=0; k<word.length; k++){
									if(!((LineItem) lines.lastElement()).addText(word[k], fontColor, bgColor, fontStyle, fontSize)){
										lines.addElement(new LineItem());
										((LineItem) lines.lastElement()).addText(word[k], fontColor, bgColor, fontStyle, fontSize);
									}
								}
							//	((LineItem) lines.lastElement()).addText(" ", fontColor, bgColor, fontStyle, fontSize);
							}
						}
					}
				}
			}
			getHeight();
		}
		public void addText(String text, SampleFont sampleFont){
			String[] strings, words;
			
			this.text += text;
			if(lines.isEmpty()) lines.addElement(new LineItem());

			text = Utils.replace(text, "\r\n", "\n\n");
			text = Utils.replace(text, "\r", "\n");
			
			strings = Utils.splitString(text, "\n");
			for(int i=0; i<strings.length; i++){
				if(strings[i].equals("")) {
					lines.addElement(new LineItem());
					((LineItem) lines.lastElement()).addText(" ", sampleFont);
				}else{
					if(strings.length > 1)lines.addElement(new LineItem());
					words = Utils.splitString(strings[i], " ");
					for(int j=0; j<words.length; j++){
						if(!((LineItem) lines.lastElement()).addText(words[j]+" ", sampleFont)){
							lines.addElement(new LineItem());
							if(words[j].length() == 0)((LineItem) lines.lastElement()).addText(" ", sampleFont);
							else if(!((LineItem) lines.lastElement()).addText(words[j]+" ", sampleFont)){
								String[] word = Utils.splitString(words[j], sampleFont, textListWidth-10);
								for(int k=0; k<word.length; k++){
									if(!((LineItem) lines.lastElement()).addText(word[k], sampleFont)){
										lines.addElement(new LineItem());
										((LineItem) lines.lastElement()).addText(word[k], sampleFont);
									}
								}
								((LineItem) lines.lastElement()).addText(" ", sampleFont);
							}
						}
					}
				}
			}
			getHeight();
		}
		public void paint(Graphics g, int left, int top){
			left += 3;
			if(!lines.isEmpty())
				for(int i=0; i<lines.size(); i++){
//					if(top + ((LineItem) lines.elementAt(i)).height > 0)
					((LineItem) lines.elementAt(i)).paint(g, left, top);
					top += ((LineItem) lines.elementAt(i)).height; 
				}
		}
		private void getHeight(){
			height = 0;
			for(int i=0; i<lines.size(); i++)
				height += ((LineItem) lines.elementAt(i)).height;
		}
	}
//	=============MAIN CLASS===================	
	public TextList(int textListWidth, int textListHeight, int fontSize, int cursorStyle, ImageList images){
		this.textListWidth = textListWidth - 3;
		this.textListHeight = textListHeight - 3;
		this.fontSize = fontSize;
		this.cursorStyle = cursorStyle;
		this.images = images;
		
		if(fontSize<0) this.sampleFont = new SampleFont("/fonts/font.bin");

		items = new Vector();
	}
//	================================
	public void setAttributes(int textListWidth, int textListHeight, int fontSize, int cursorStyle, ImageList images){
		this.textListWidth = textListWidth - 3;
		this.textListHeight = textListHeight - 3;
		this.fontSize = fontSize;
		this.cursorStyle = cursorStyle;
	}
	public void setImageList(ImageList imagelist){
		this.images = imagelist;
	}
//	================================
	public int getItemType(int index){
		return ((ListItem) items.elementAt(index)).type;
	}
	public String getItemText(int index){
		if(index>-1 && index<items.size()) return ((ListItem) items.elementAt(index)).text;
		else return "";
	}
//	================================
	public void insertImage(Image image, int index){
		items.insertElementAt(new ListItem(), index);
		((ListItem) items.elementAt(index)).addImage(image);
	}
	public void insertImage(int imageIndex, int index){
		if(images!=null && imageIndex>-1 && imageIndex<images.getSize()){
			items.insertElementAt(new ListItem(), index);
			((ListItem) items.elementAt(index)).addImage(imageIndex);
		}
	}
	public void insertText(String text, int index){
		items.insertElementAt(new ListItem(), index);
		((ListItem) items.elementAt(index)).addText(text, 0x000000, 0xffffff, Font.STYLE_PLAIN, fontSize);
	}
	public void addImage(Image image){
		items.addElement(new ListItem());
		((ListItem) items.lastElement()).addImage(image);
	}
	public void addImage(Image image, int itemIndex){
		if(itemIndex<items.size()){
			((ListItem) items.elementAt(itemIndex)).addImage(image);
		}else addImage(image);
	}
	public void addImage(int imageIndex){
		if(images!=null && imageIndex>-1 && imageIndex<images.getSize()){
			items.addElement(new ListItem());
			((ListItem) items.lastElement()).addImage(imageIndex);
		}
	}
	public void addImage(int imageIndex, int itemIndex){
		if(images!=null && imageIndex>-1 && imageIndex<images.getSize()){
			if(itemIndex<items.size()){
				((ListItem) items.elementAt(itemIndex)).addImage(imageIndex);
			}else addImage(imageIndex);
		}
	}
	public void addText(String text){
		addText(text, 0x000000, 0xffffff, Font.STYLE_PLAIN, fontSize);
	}
	public void addText(String text, int itemIndex){
		addText(text, 0x000000, 0xffffff, Font.STYLE_PLAIN, fontSize, itemIndex);
	}
	public void addText(String text, int fontColor, int bgColor, int fontStyle, int fontSize){
		addText(text, fontColor, bgColor, fontStyle, fontSize, items.size());
	}
	public void addURL(String URL, int bgColor){
		items.addElement(new ListItem());
		((ListItem) items.lastElement()).type = TYPE_URL;
		((ListItem) items.lastElement()).addText(URL, 0x0000ff, bgColor, Font.STYLE_UNDERLINED, fontSize);
	}
	public void addText(String text, int fontColor, int bgColor, int fontStyle, int fontSize, int itemIndex){
		if(text.length()>0){
			if(itemIndex<items.size()){
				if(fontSize==-1){
					((ListItem) items.elementAt(itemIndex)).addText(text, sampleFont);
				}else{
					((ListItem) items.elementAt(itemIndex)).addText(text, fontColor, bgColor, fontStyle, fontSize);
				}
			}else{
				items.addElement(new ListItem());
				if(fontSize==-1){
					((ListItem) items.elementAt(itemIndex)).addText(text, sampleFont);
				}else{
					((ListItem) items.elementAt(itemIndex)).addText(text, fontColor, bgColor, fontStyle, fontSize);
				}
			}
		}
	}
	public void addTextWithURL(String text){
		addTextWithURL(text, 0x000000, 0xffffff, Font.STYLE_PLAIN, fontSize, items.size());
	}
	public void addTextWithURL(String text, int fontColor, int bgColor, int fontStyle, int fontSize, int itemIndex){
		if(text.length()>0){
			int URLIndex = text.toLowerCase().indexOf("http://");
			if(URLIndex == -1) URLIndex = text.toLowerCase().indexOf("ftp://");
			if(URLIndex == -1) URLIndex = text.toLowerCase().indexOf("mailto:");
			if(URLIndex == -1) URLIndex = text.toLowerCase().indexOf("www.");
			if(URLIndex == -1) URLIndex = text.toLowerCase().indexOf("https://");
			if(URLIndex>-1){
				String temp = text;
				if(URLIndex>1){
					temp = temp.substring(0, URLIndex-1);
					addTextWithURL(temp, fontColor, bgColor, fontStyle, fontSize, itemIndex);
				}
				temp = text.substring(URLIndex);
		
				int splitIndex;
				for(splitIndex=0; splitIndex<temp.length(); splitIndex++){
					char ch = temp.charAt(splitIndex);
					if (!((ch >= 0x2d && ch <= 0x3a) || 
						(ch >= 0x40 && ch <= 0x5a)   || 
						(ch >= 0x61 && ch <= 0x7a)   || 
						 ch == 0x3f || ch == 0x3d    || 
						 ch == 0x5f ||
						 ch == 0x26 || ch == 0x25)){    
						break;
					}
				}
				
				if(splitIndex>-1){
					addURL(temp.substring(0, splitIndex), bgColor);
					temp = temp.substring(splitIndex);
					addTextWithURL(temp, fontColor, bgColor, fontStyle, fontSize, items.size());
				}else addURL(temp, bgColor);
			}else if(itemIndex<items.size()){
				((ListItem) items.elementAt(itemIndex)).addText(text, fontColor, bgColor, fontStyle, fontSize);
			}else{
				items.addElement(new ListItem());
				((ListItem) items.elementAt(itemIndex)).addText(text, fontColor, bgColor, fontStyle, fontSize);
			}
		}
	}
//	================================
	public void addItem(){
		items.addElement(new ListItem());
	}
	public void deleteItem(int itemIndex){
		items.removeElementAt(itemIndex);
	}
//	================================
	public void up(){
		if(currentItem > 0 || topPosition < 0){
			if(currentItem > topItem){
				currentItem--;
			}else{ 
				if(topPosition < 0){
					topPosition += 24;
					if(topPosition > 0) topPosition =0;
				}else{
					topPosition = 0;
					topItem--;
					currentItem--;
					if(((ListItem) items.elementAt(currentItem)).height > textListHeight) 
						topPosition = textListHeight - ((ListItem) items.elementAt(currentItem)).height;
				}
			}
		}
	}
	public void down(){
		if(currentItem < items.size()){
			if(getItemsHeight(topItem, currentItem)+topPosition < textListHeight){
				if(currentItem < items.size()-1)currentItem++;
			}
			if(getItemsHeight(topItem, currentItem)+topPosition >= textListHeight){
				if(topPosition + ((ListItem) items.elementAt(currentItem)).height > textListHeight){
					topPosition -= 24;
					if(topPosition + ((ListItem) items.elementAt(currentItem)).height < textListHeight)
						topPosition = textListHeight - ((ListItem) items.elementAt(currentItem)).height-2;
				}else{
					if(getItemsHeight(topItem, currentItem)+topPosition == textListHeight){
						if(currentItem < items.size()-1)currentItem++;
						if(topPosition + ((ListItem) items.elementAt(currentItem)).height > textListHeight){
							topPosition -= 24;
						}else{
							topPosition -= getItemsHeight(topItem, currentItem)+topPosition - textListHeight+2;
						}
					}else{
						topPosition -= getItemsHeight(topItem, currentItem)+topPosition - textListHeight+2;
					}
				}
			}
			while(topPosition + ((ListItem) items.elementAt(topItem)).height <= 0 && topItem < items.size()-1){
				topPosition += ((ListItem) items.elementAt(topItem)).height;
				topItem++;
			}
		}
	}
	public void pageDown(){
		if(getItemsHeight(0, items.size()-1) > textListHeight - 5){
			if(getItemsHeight(topItem, items.size()-1)+topPosition >= textListHeight*2){
				topPosition -= textListHeight;
				while(topPosition + ((ListItem) items.elementAt(topItem)).height < 0 & topItem < items.size()-2){
					topPosition += ((ListItem) items.elementAt(topItem)).height;
					topItem++;
				}
			}else{
				topPosition -= getItemsHeight(topItem, items.size()-1)+topPosition - textListHeight;
				while(topPosition + ((ListItem) items.elementAt(topItem)).height <= 0 & topItem < items.size()-2){
					topPosition += ((ListItem) items.elementAt(topItem)).height;
					topItem++;
				}
			}
			while(topPosition + getItemsHeight(topItem, currentItem) < textListHeight && currentItem < items.size()){
				currentItem++;
			}
			topItem = currentItem;
			topPosition -= 1;
		}
	}
	public void pageUp(){
		int oldTopItem = topItem;
		if(getItemsHeight(0, topItem) > textListHeight){
			topPosition = 0;
			while(getItemsHeight(topItem, oldTopItem) < textListHeight & topItem > 0){
				topItem--;
			}
		}else{
			topPosition = 0;
			topItem = 0;
		}
		currentItem = topItem;
	}
	public void end(){
		topItem = 0;
		currentItem = items.size()-1;
		if(getItemsHeight(0, currentItem) > textListHeight){
			topPosition = textListHeight - getItemsHeight(0, currentItem)-2;
			while(topPosition + ((ListItem) items.elementAt(topItem)).height <= 0 & topItem < items.size()-2){
				topPosition += ((ListItem) items.elementAt(topItem)).height;
				topItem++;
			}
		}
	}
	public void home(){
		topPosition = 0;
		topItem = currentItem = 0;
	}
//	================================
	public void clear(){
		items.removeAllElements();
		topPosition = 0;
		topItem = 0;
		currentItem = 0;
	}
//	================================
	private int getItemsHeight(int from, int to){
		int height = 0;
		for(int i=from; i <= to; i++) height += ((ListItem) items.elementAt(i)).height;
		return height;
	}
//	================================
	private void drawItems(Graphics g, int left, int top){
		top+=2;
		if(!items.isEmpty()){
			int drawedItem = topItem,
				oldTop = top - topPosition;
		
			while(top-oldTop < textListHeight & drawedItem < items.size()){
				if(drawedItem == currentItem & cursorStyle == CURSOR_RECT){
					g.setColor(219, 225, 231);
					g.fillRect(left+3, top, textListWidth-6, ((ListItem) items.elementAt(drawedItem)).height);
					
					g.setColor(180, 180, 180);
					
					g.drawLine(left+2, top, left+2, top + ((ListItem) items.elementAt(drawedItem)).height-1);

					g.drawLine(left+3, top-1, left + textListWidth-4, top - 1);
					g.drawLine(left+3, top + ((ListItem) items.elementAt(drawedItem)).height, left + textListWidth-4, top + ((ListItem) items.elementAt(drawedItem)).height);

					g.drawLine(left + textListWidth-3, top, left + textListWidth-3, top + ((ListItem) items.elementAt(drawedItem)).height-1);
				}
				((ListItem) items.elementAt(drawedItem)).paint(g, left, top);
				top += ((ListItem) items.elementAt(drawedItem)).height;
				drawedItem++;
			}
		}
	}
	private void drawScrollBar(Graphics g, int left, int top){
		int x = left + textListWidth;
		g.setColor(180, 180, 180);
		g.drawRect(x-1, top, 4, textListHeight);
		g.setColor(230, 230, 230);
		g.drawRect(x, top, 1, textListHeight);
		
		if(!items.isEmpty()){
			int n = (getItemsHeight(0, items.size()-1))*1000/textListHeight;
			int y = top;
			if(n != 0) y +=(-topPosition+getItemsHeight(0, topItem)-((ListItem) items.elementAt(topItem)).height)*1000/n;
			int scrollBarHeight=0;
			if(n != 0) scrollBarHeight = textListHeight*1000/n;

		if(scrollBarHeight < textListHeight){
			g.setColor(170, 200, 250);
			g.fillRect(x, y, 2, scrollBarHeight);
			g.setColor(100, 130, 180);
			g.fillRect(x+1, y, 1, scrollBarHeight);
			g.setColor(100, 100, 100);
			g.drawRect(x-1, y, 3, scrollBarHeight);
		}
		}
	}
//	================================	
	public void paint(Graphics g, int left, int top){
		int clipX = g.getClipX(),
			clipY = g.getClipY(),
			clipWidth  = g.getClipWidth(),
			clipHeight = g.getClipHeight();

		g.setClip(left, top, textListWidth+3, textListHeight+1);
		g.setColor(255, 255, 255);
		g.fillRect(left, top+1, textListWidth+3, textListHeight);
		drawItems(g , left, top+topPosition);
		drawScrollBar(g, left, top+1);
		g.setClip(clipX, clipY, clipWidth, clipHeight);
		g.setColor(100, 100, 100);
		g.drawRect(left, top, textListWidth+2, textListHeight+1);
	}
//	================================
}
